@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 try:
     from zope.app.event.objectevent import objectEventNotify # BBB
 except ImportError, e:
@@ -35,6 +36,7 @@ from node.ext.ldap import (
     ONELEVEL,
     LDAPSession,
 )
+from node.ext.ldap.base import escape
 from node.ext.ldap.interfaces import ILDAPStorage
 from node.ext.ldap.events import (
     LDAPNodeCreatedEvent,
@@ -48,6 +50,7 @@ from node.ext.ldap.filter import (
     LDAPDictFilter,
     LDAPRelationFilter,
 )
+from node.ext.ldap.schema import LDAPSchemaInfo
 from ldap.functions import explode_dn
 from ldap import (
     MOD_ADD,
@@ -77,11 +80,22 @@ class AttributesPart(Part):
         
         self.clear()
         
+        attrlist = ['*']
+        
+        # XXX: operational attributes
+        #if self.session._props.operationalAttributes:
+        #    attrlist.append('+')
+        
+        # XXX: if memberOf support enabled
+        #if self.session._props.memberOfSupport:
+        #    attrlist.append('memberOf')
+        
         # fetch our node with all attributes
         entry = self.parent.ldap_session.search(
                 scope=BASE,
                 baseDN=self.parent.DN,
-                force_reload=self.parent._reload
+                force_reload=self.parent._reload,
+                attrlist=attrlist,
                 )
         
         if len(entry) != 1:
@@ -91,11 +105,7 @@ class AttributesPart(Part):
         
         attrs = entry[0][1]
         for key, item in attrs.items():
-            # XXX: future: self[key] = item, always!
-            # XXX: maybe we want to keep behaviour, but validate against
-            #      schema?
-            if len(item) == 1 \
-              and key not in ('member', 'uniqueMember', 'memberUid'):
+            if len(item) == 1 and not self.is_multivalued(key):
                 self[key] = item[0]
             else:
                 self[key] = item
@@ -123,6 +133,14 @@ class AttributesPart(Part):
         if self.parent._action not in [ACTION_ADD, ACTION_DELETE]:
             self.parent._action = ACTION_MODIFY
             self.parent.changed = True
+        
+    @default    
+    def is_binary(self, name):
+        return name in self.parent.root._binary_attributes
+
+    @default    
+    def is_multivalued(self, name):
+        return name in self.parent.root._multivalued_attributes
 
 
 class LDAPNodeAttributes(NodeAttributes):
@@ -167,13 +185,17 @@ class LDAPStorage(OdictStorage):
         self._reload = False
         self._init_keys()
         if props:
+            # only at root node
             self._ldap_session = LDAPSession(props)
             self._ldap_session.baseDN = self.DN
-        
+            self._ldap_schema_info = LDAPSchemaInfo(props)
+            self._multivalued_attributes = props.multivalued_attributes
+            self._binary_attributes = props.binary_attributes
+
         # XXX: make them public
         self._key_attr = 'rdn'
         self._rdn_attr = None
-        
+
         # search related defaults
         self.search_scope = ONELEVEL
         self.search_filter = None
@@ -325,7 +347,7 @@ class LDAPStorage(OdictStorage):
     def __repr__(self):
         # Doctest fails if we output utf-8
         try:
-            dn =  self.DN.encode('ascii', 'replace') or '(dn not set)'
+            dn = self.DN.encode('ascii', 'replace') or '(dn not set)'
         except KeyError, e:
             dn = '(dn not available yet)' 
         if self.parent is None:
@@ -430,7 +452,13 @@ class LDAPStorage(OdictStorage):
         # fetch also the key attribute
         if not self._key_attr == 'rdn':
             attrset.add(self._key_attr)
-
+        
+        # XXX: Make this work for string queries and move to LDADCommunicator
+        #if self.ldap_session._props.escape_queries:
+        #    if criteria is not None:
+        #        for key, val in criteria.items():
+        #            criteria[key] = escape(val)
+        
         # Create queryFilter from all filter definitions
         # filter for this search ANDed with the default filters defined on self
         search_filter = LDAPFilter(queryFilter)
@@ -455,7 +483,7 @@ class LDAPStorage(OdictStorage):
         # would it be better to fail? (see also __iter__ secondary key)
         if self._key_attr != 'rdn' and self._key_attr not in _filter:
             _filter &= '(%s=*)' % (self._key_attr,)
-
+        
         # perform the backend search
         matches = self.ldap_session.search(
             str(_filter),
@@ -605,9 +633,8 @@ class LDAPStorage(OdictStorage):
             try:
                 seckey = attrs[seckey_attr]
             except KeyError:
-                raise KeyError(
-                    u"Secondary key '%s' missing on: %s." % \
-                        (seckey_attr, attrs['dn']))
+                # no sec key found, skip
+                continue
             else:
                 if isinstance(seckey, list):
                     if len(seckey) != 1:
@@ -668,6 +695,12 @@ class LDAPStorage(OdictStorage):
         del self.parent._keys[self.name]
         self.ldap_session.delete(self.DN)
 
+    @default
+    @property
+    def schema_info(self):
+        if self.parent is not None:
+            return self.root._ldap_schema_info
+        return self._ldap_schema_info
 
 class LDAPNode(object):
     __metaclass__ = plumber
