@@ -1,8 +1,10 @@
+# -*- coding: utf-8 -*-
 import os
 import shutil
 import subprocess
 import tempfile
 import time
+import logging
 from odict import odict
 from plone.testing import Layer, zca
 from pkg_resources import resource_filename
@@ -23,11 +25,12 @@ try:
     SLAPDURIS = os.environ['SLAPD_URIS']
     LDAPADDBIN = os.environ['LDAP_ADD_BIN']
     LDAPDELETEBIN = os.environ['LDAP_DELETE_BIN']
-except KeyError:                                            #pragma NO COVERAGE
-    raise RuntimeError(                                     #pragma NO COVERAGE
-        u"Environment variables SLAPD_BIN, SLAPD_URIS, "    #pragma NO COVERAGE
-        u"LDAP_ADD_BIN, LDAP_DELETE_BIN needed.")           #pragma NO COVERAGE
-
+    LDAPSUFFIX = os.environ.get('LDAP_SUFFIX', None) or "dc=my-domain,dc=com"
+except KeyError, e:                                          #pragma NO COVERAGE
+    logging.exception(                                       #pragma NO COVERAGE
+        u"Environment variables SLAPD_BIN, SLAPD_URIS, "     #pragma NO COVERAGE
+        u"LDAP_ADD_BIN, LDAP_DELETE_BIN needed.")            #pragma NO COVERAGE 
+    exit(1)                                                  #pragma NO COVERAGE
 
 def resource(string):
     return resource_filename(__name__, string)
@@ -46,7 +49,8 @@ def read_env(layer):
     layer['confdir'] = tmpdir
     layer['dbdir'] = "%s/openldap-data" % (layer['confdir'],)
     layer['slapdconf'] = "%s/slapd.conf" % (layer['confdir'],)
-    layer['binddn'] = "cn=Manager,dc=my-domain,dc=com"
+    layer['binddn'] = "cn=Manager,%s" % LDAPSUFFIX
+    layer['suffix'] = LDAPSUFFIX
     layer['bindpw'] = "secret"
     print tmpdir
 
@@ -58,12 +62,14 @@ pidfile		%(confdir)s/slapd.pid
 argsfile	%(confdir)s/slapd.args
 
 database	bdb
-suffix		"dc=my-domain,dc=com"
+suffix		"%(suffix)s"
 rootdn		"%(binddn)s"
 rootpw		%(bindpw)s
 directory	%(dbdir)s
 # Indices to maintain
 index	objectClass	eq
+
+overlay memberof
 """
 
 
@@ -83,6 +89,7 @@ class SlapdConf(Layer):
         knows
         """
         read_env(self)
+        suffix = self['suffix']
         binddn = self['binddn']
         bindpw = self['bindpw']
         confdir = self['confdir']
@@ -98,7 +105,8 @@ class SlapdConf(Layer):
                     bindpw=bindpw,
                     confdir=confdir,
                     dbdir=dbdir,
-                    schema=schema
+                    schema=schema,
+                    suffix=suffix,
                     ))
         os.mkdir(dbdir)
         self['props'] = props
@@ -109,16 +117,18 @@ class SlapdConf(Layer):
         """
         read_env(self)
         shutil.rmtree(self['confdir'])
+        del self['confdir']
         print "SlapdConf torn down."
 
 
-schema = (
+schema = [
     resource('schema/core.schema'),
     resource('schema/cosine.schema'),
     resource('schema/inetorgperson.schema'),
     resource('schema/nis.schema'),
     resource('schema/samba.schema'),
-    )
+    resource('schema/ad.dummy.schema'),
+]
 
 SLAPD_CONF = SlapdConf(schema)
 
@@ -149,6 +159,7 @@ class Slapd(LDAPLayer):
         read_env(self)
         cmd = [self.slapdbin, '-f', self['slapdconf'], '-h', self['uris']]
         cmd += args
+        print ' '.join(cmd)
         self.slapd = subprocess.Popen(cmd)
         time.sleep(1)
         print "done."
@@ -202,9 +213,8 @@ class Ldif(LDAPLayer):
     def setUp(self):
         """run ldapadd for list of ldifs
         """
-        # Create a new global registry
         if not os.environ.get('node.ext.ldap.testldap.skip_zca_hook'):
-            zca.pushGlobalRegistry()        
+            zca.pushGlobalRegistry()
 
         read_env(self)
         self['ucfg'] = self.ucfg
@@ -233,10 +243,19 @@ class Ldif(LDAPLayer):
         for key in ('ucfg', 'gcfg'):
             if key in self:
                 del self[key]
-        # Pop the global registry
+        
         if not os.environ.get('node.ext.ldap.testldap.skip_zca_hook'):
-            zca.popGlobalRegistry()
-                
+            # XXX: fails to pop global registry in Zope 2. Why?
+            try:
+                zca.popGlobalRegistry()
+                print 80 * '*'
+                print "pop global registry"
+            except Exception, e:
+                print 80 * '*'
+                print "pop global registry failed"
+                print e
+                print 80 * '*'
+
 
 ldif_layer = odict()
             
@@ -434,28 +453,6 @@ LDIF_groupOfNames = Ldif(
 ldif_layer['groupOfNames'] = LDIF_groupOfNames
 
 
-LDIF_groupOfNames_add = Ldif(
-    resource('ldifs/groupOfNames_add.ldif'),
-    bases=(LDIF_groupOfNames,),
-    name="LDIF_groupOfNames_add",
-    ucfg=UsersConfig(
-        baseDN='ou=users,ou=groupOfNames,dc=my-domain,dc=com',
-        attrmap=groupOfNamesUcfgAttrmap,
-        scope=SUBTREE,
-        queryFilter='(objectClass=inetOrgPerson)',
-        objectClasses=['person', 'inetOrgPerson'],
-        ),
-    gcfg=GroupsConfig(
-        baseDN='ou=groups,ou=groupOfNames,dc=my-domain,dc=com',
-        attrmap=groupOfNamesGcfgAttrmap,
-        scope=SUBTREE,
-        queryFilter='(objectClass=groupOfNames)',
-        objectClasses=['groupOfNames'],
-        ),
-    )
-ldif_layer['groupOfNames_add'] = LDIF_groupOfNames_add
-
-
 LDIF_groupOfNames_10_10 = Ldif(
     resource('ldifs/groupOfNames_10_10.ldif'),
     bases=(LDIF_base,),
@@ -500,28 +497,6 @@ LDIF_groupOfNames_100_100 = Ldif(
 ldif_layer['groupOfNames_100_100'] = LDIF_groupOfNames_100_100
 
 
-LDIF_groupOfNames_100_100_add = Ldif(
-    resource('ldifs/groupOfNames_100_100_add.ldif'),
-    bases=(LDIF_groupOfNames_100_100,),
-    name="LDIF_groupOfNames_100_100_add",
-    ucfg=UsersConfig(
-        baseDN='ou=users,ou=groupOfNames_100_100,dc=my-domain,dc=com',
-        attrmap=groupOfNamesUcfgAttrmap,
-        scope=SUBTREE,
-        queryFilter='(objectClass=inetOrgPerson)',
-        objectClasses=['person', 'inetOrgPerson'],
-        ),
-    gcfg=GroupsConfig(
-        baseDN='ou=groups,ou=groupOfNames_100_100,dc=my-domain,dc=com',
-        attrmap=groupOfNamesGcfgAttrmap,
-        scope=SUBTREE,
-        queryFilter='(objectClass=groupOfNames)',
-        objectClasses=['groupOfNames'],
-        ),
-    )
-ldif_layer['groupOfNames_100_100_add'] = LDIF_groupOfNames_100_100_add
-
-
 LDIF_groupOfNames_300_300 = Ldif(
     resource('ldifs/groupOfNames_300_300.ldif'),
     bases=(LDIF_base,),
@@ -544,28 +519,6 @@ LDIF_groupOfNames_300_300 = Ldif(
 ldif_layer['groupOfNames_300_300'] = LDIF_groupOfNames_300_300
 
 
-LDIF_groupOfNames_300_300_add = Ldif(
-    resource('ldifs/groupOfNames_300_300_add.ldif'),
-    bases=(LDIF_groupOfNames_300_300,),
-    name="LDIF_groupOfNames_300_300_add",
-    ucfg=UsersConfig(
-        baseDN='ou=users,ou=groupOfNames_300_300,dc=my-domain,dc=com',
-        attrmap=groupOfNamesUcfgAttrmap,
-        scope=SUBTREE,
-        queryFilter='(objectClass=inetOrgPerson)',
-        objectClasses=['person', 'inetOrgPerson'],
-        ),
-    gcfg=GroupsConfig(
-        baseDN='ou=groups,ou=groupOfNames_300_300,dc=my-domain,dc=com',
-        attrmap=groupOfNamesGcfgAttrmap,
-        scope=SUBTREE,
-        queryFilter='(objectClass=groupOfNames)',
-        objectClasses=['groupOfNames'],
-        ),
-    )
-ldif_layer['groupOfNames_300_300_add'] = LDIF_groupOfNames_300_300_add
-
-
 LDIF_groupOfNames_700_700 = Ldif(
     resource('ldifs/groupOfNames_700_700.ldif'),
     bases=(LDIF_base,),
@@ -588,28 +541,6 @@ LDIF_groupOfNames_700_700 = Ldif(
 ldif_layer['groupOfNames_700_700'] = LDIF_groupOfNames_700_700
 
 
-LDIF_groupOfNames_700_700_add = Ldif(
-    resource('ldifs/groupOfNames_700_700_add.ldif'),
-    bases=(LDIF_groupOfNames_700_700,),
-    name="LDIF_groupOfNames_700_700_add",
-    ucfg=UsersConfig(
-        baseDN='ou=users,ou=groupOfNames_700_700,dc=my-domain,dc=com',
-        attrmap=groupOfNamesUcfgAttrmap,
-        scope=SUBTREE,
-        queryFilter='(objectClass=inetOrgPerson)',
-        objectClasses=['person', 'inetOrgPerson'],
-        ),
-    gcfg=GroupsConfig(
-        baseDN='ou=groups,ou=groupOfNames_700_700,dc=my-domain,dc=com',
-        attrmap=groupOfNamesGcfgAttrmap,
-        scope=SUBTREE,
-        queryFilter='(objectClass=groupOfNames)',
-        objectClasses=['groupOfNames'],
-        ),
-    )
-ldif_layer['groupOfNames_700_700_add'] = LDIF_groupOfNames_700_700_add
-
-
 LDIF_groupOfNames_1000_1000 = Ldif(
     resource('ldifs/groupOfNames_1000_1000.ldif'),
     bases=(LDIF_base,),
@@ -632,28 +563,6 @@ LDIF_groupOfNames_1000_1000 = Ldif(
 ldif_layer['groupOfNames_1000_1000'] = LDIF_groupOfNames_1000_1000
 
 
-LDIF_groupOfNames_1000_1000_add = Ldif(
-    resource('ldifs/groupOfNames_1000_1000_add.ldif'),
-    bases=(LDIF_groupOfNames_1000_1000,),
-    name="LDIF_groupOfNames_1000_1000_add",
-    ucfg=UsersConfig(
-        baseDN='ou=users,ou=groupOfNames_1000_1000,dc=my-domain,dc=com',
-        attrmap=groupOfNamesUcfgAttrmap,
-        scope=SUBTREE,
-        queryFilter='(objectClass=inetOrgPerson)',
-        objectClasses=['person', 'inetOrgPerson'],
-        ),
-    gcfg=GroupsConfig(
-        baseDN='ou=groups,ou=groupOfNames_1000_1000,dc=my-domain,dc=com',
-        attrmap=groupOfNamesGcfgAttrmap,
-        scope=SUBTREE,
-        queryFilter='(objectClass=groupOfNames)',
-        objectClasses=['groupOfNames'],
-        ),
-    )
-ldif_layer['groupOfNames_1000_1000_add'] = LDIF_groupOfNames_1000_1000_add
-
-
 # Users and groups (posix)
 
 posixGroupsUcfgAttrmap = {
@@ -696,6 +605,28 @@ LDIF_posixGroups = Ldif(
 ldif_layer['posixGroups'] = LDIF_posixGroups
 
 
+LDIF_posixGroups_10_10 = Ldif(
+    resource('ldifs/posixGroups_10_10.ldif'),
+    bases=(LDIF_base,),
+    name="LDIF_posixGroups_10_10",
+    ucfg=UsersConfig(
+        baseDN='ou=users,ou=posixGroups_10_10,dc=my-domain,dc=com',
+        attrmap=posixGroupsUcfgAttrmap,
+        scope=ONELEVEL,
+        queryFilter='(objectClass=posixAccount)',
+        objectClasses=['account', 'posixAccount'],
+        ),
+    gcfg=GroupsConfig(
+        baseDN='ou=groups,ou=posixGroups_10_10,dc=my-domain,dc=com',
+        attrmap=posixGroupsGcfgAttrmap,
+        scope=ONELEVEL,
+        queryFilter='(objectClass=posixGroup)',
+        objectClasses=['posixGroup'],
+        ),
+    )
+ldif_layer['posixGroups_10_10'] = LDIF_posixGroups_10_10
+
+
 # users (samba)
 
 sambaUsersUcfgAttrmap = {
