@@ -1,4 +1,8 @@
+# -*- coding: utf-8 -*-
 import ldap
+import types
+import time
+import logging
 from plumber import (
     plumber,
     plumb,
@@ -35,7 +39,11 @@ from node.ext.ldap.interfaces import (
     ILDAPGroupsConfig as IGroupsConfig,
     ILDAPUsersConfig as IUsersConfig,
 )
-from node.ext.ldap.scope import ONELEVEL
+from node.ext.ldap.scope import (
+    BASE,
+    ONELEVEL,
+)
+from node.ext.ldap.base import decode_utf8
 from node.ext.ldap._node import LDAPNode
 from node.ext.ldap.ugm.defaults import creation_defaults
 from node.ext.ldap.ugm.samba import (
@@ -44,31 +52,50 @@ from node.ext.ldap.ugm.samba import (
 )
 
 
+logger = logging.getLogger('node.ext.ldap')
+
+# group member format
 FORMAT_DN = 0
 FORMAT_UID = 1
 
+# expiration unit
+EXPIRATION_DAYS = 0
+EXPIRATION_SECONDS = 1
+
+
+class AccountExpired(object):
+    
+    def __nonzero__(self):
+        return False
+    
+    def __repr__(self):
+        return 'ACCOUNT_EXPIRED'
+    
+    __str__ = __repr__
+    
+ACCOUNT_EXPIRED = AccountExpired()
+
 
 class PrincipalsConfig(object):
-    """Superclass for UsersConfig, GroupsConfig and RolesConfig (later)
-    """
     
-    def __init__(self,
-            baseDN='',
-            attrmap={},
-            scope=ONELEVEL,
-            queryFilter='',
-            objectClasses=[],
-            #member_relation='',
-            defaults={},
-            strict=True):
+    def __init__(self, baseDN='', attrmap={}, scope=ONELEVEL, queryFilter='',
+                 objectClasses=[], defaults={}, strict=True,
+                 memberOfSupport=False, expiresAttr=None,
+                 expiresUnit=EXPIRATION_DAYS):
         self.baseDN = baseDN
         self.attrmap = attrmap
         self.scope = scope
         self.queryFilter = queryFilter
         self.objectClasses = objectClasses
-        #self.member_relation = member_relation
         self.defaults = defaults
         self.strict = strict
+        self.memberOfSupport = memberOfSupport
+        # XXX: currently expiresAttr only gets considered for user
+        #      authentication group and role expiration is not implemented yet.
+        self.expiresAttr = expiresAttr
+        self.expiresUnit = expiresUnit
+        # XXX: member_relation
+        #self.member_relation = member_relation
 
 
 class UsersConfig(PrincipalsConfig):
@@ -139,6 +166,21 @@ class LDAPPrincipal(AliasedPrincipal):
     @property
     def changed(self):
         return self.context.changed
+    
+    @default
+    @property
+    def member_of_attr(self):
+        """memberOf is in openldap realized as overlay and in Active
+        Directory also computed. In case of openldap this attribute is not
+        delivered in LDAP response unless explicitly queried. Thus a separate
+        property is used to query memberOf information explicit.
+        """
+        entry = self.context.ldap_session.search(
+            scope=BASE,
+            baseDN=self.context.DN,
+            force_reload=self.context._reload,
+            attrlist=['memberOf'])
+        return entry[0][1].get('memberOf', list())
 
 
 class LDAPUser(LDAPPrincipal, UgmUser):
@@ -147,18 +189,28 @@ class LDAPUser(LDAPPrincipal, UgmUser):
     @property
     def groups(self):
         groups = self.parent.parent.groups
-        member_format = groups._member_format
-        attribute = groups._member_attribute
-        if member_format == FORMAT_DN:
-            criteria = { attribute: self.context.DN }
-        elif member_format == FORMAT_UID:
-            criteria = { attribute: self.context.attrs['uid'] }
-        # if roles configuration points to child of groups container, and
-        # group configuration has search scope SUBTREE, and groups are
-        # specified by the same criteria as roles, the search returns the 
-        # role id's as well.
-        # XXX: such edge cases should be resolved at UGM init time
-        res = groups.context.search(criteria=criteria)
+        if self.parent.parent.ucfg.memberOfSupport:
+            res = list()
+            for dn in self.member_of_attr:
+                try:
+                    res.append(groups.idbydn(dn))
+                except KeyError:
+                    # happens if DN is returned which does not fit the groups
+                    # base DN.
+                    pass
+        else:
+            member_format = groups._member_format
+            attribute = groups._member_attribute
+            if member_format == FORMAT_DN:
+                criteria = { attribute: self.context.DN }
+            elif member_format == FORMAT_UID:
+                criteria = { attribute: self.context.attrs['uid'] }
+            # if roles configuration points to child of groups container, and
+            # group configuration has search scope SUBTREE, and groups are
+            # specified by the same criteria as roles, the search returns the 
+            # role id's as well.
+            # XXX: such edge cases should be resolved at UGM init time
+            res = groups.context.search(criteria=criteria)
         ret = list()
         for uid in res:
             ret.append(groups[uid])
@@ -179,6 +231,7 @@ class LDAPGroupMapping(Part):
     
     @extend
     def __getitem__(self, key):
+        key = decode_utf8(key)
         if key not in self:
             raise KeyError(key)
         return self.related_principals(key)[key]
@@ -186,6 +239,7 @@ class LDAPGroupMapping(Part):
     @extend
     @locktree
     def __delitem__(self, key):
+        key = decode_utf8(key)
         if key not in self:
             raise KeyError(key)
         if self._member_format == FORMAT_DN:
@@ -206,6 +260,7 @@ class LDAPGroupMapping(Part):
     
     @extend
     def __contains__(self, key):
+        key = decode_utf8(key)
         for uid in self:
             if uid == key:
                 return True
@@ -214,6 +269,7 @@ class LDAPGroupMapping(Part):
     @default
     @locktree
     def add(self, key):
+        key = decode_utf8(key)
         if not key in self.member_ids:
             val = self.translate_key(key)
             # self.context.attrs[self._member_attribute].append won't work here
@@ -226,6 +282,20 @@ class LDAPGroupMapping(Part):
     @default
     @property
     def member_ids(self):
+        ugm = self.parent.parent
+        if ugm:
+            # XXX: roles with memberOf use rcfg!
+            gcfg = ugm.gcfg
+            if gcfg and gcfg.memberOfSupport:
+                users = ugm.users
+                criteria = {
+                    'memberOf': self.context.DN,
+                }
+                # XXX: use users instead of users.context.
+                #      Problem: Aliaed Attributes in strict mode do not know
+                #               about memberOf by default. Check for related
+                #               configuration flag and set transparently 
+                return users.context.search(criteria=criteria)
         ret = list()
         members = self.context.attrs.get(self._member_attribute, list())
         for member in members:
@@ -335,6 +405,8 @@ class LDAPPrincipals(OdictStorage):
         
         context._load_keys()
         
+        self.expiresAttr = cfg.expiresAttr
+        self.expiresUnit = cfg.expiresUnit
         self.principal_attrmap = cfg.attrmap
         self.principal_attraliaser = DictAliaser(cfg.attrmap, cfg.strict)
         self.context = context
@@ -383,6 +455,7 @@ class LDAPPrincipals(OdictStorage):
     @default
     @locktree
     def __delitem__(self, key):
+        key = decode_utf8(key)
         del self.context[key]
         try:
             del self.storage[key]
@@ -392,6 +465,7 @@ class LDAPPrincipals(OdictStorage):
     @default
     @locktree
     def __getitem__(self, key):
+        key = decode_utf8(key)
         try:
             return self.storage[key]
         except KeyError:
@@ -411,6 +485,9 @@ class LDAPPrincipals(OdictStorage):
     @default
     @locktree
     def __setitem__(self, name, vessel):
+        """XXX: mechanism for defining a target container if search scope is
+                SUBTREE
+        """
         try:
             attrs = vessel.attrs
         except AttributeError:
@@ -440,6 +517,7 @@ class LDAPPrincipals(OdictStorage):
     def invalidate(self, key=None):
         """Invalidate LDAPPrincipals.
         """
+        key = decode_utf8(key)
         self.context.invalidate(key)
         if key is None:
             self.storage.clear()
@@ -521,6 +599,7 @@ class LDAPUsers(LDAPPrincipals, UgmUsers):
     @extend
     @locktree
     def __delitem__(self, key):
+        key = decode_utf8(key)
         user = self[key]
         try:
             groups = user.groups
@@ -534,13 +613,48 @@ class LDAPUsers(LDAPPrincipals, UgmUsers):
                 user.remove_role(role)
         del self.context[key]
     
+    @default
+    def id_for_login(self, login):
+        return self.context._seckeys.get(
+            self.principal_attrmap.get('login'), {}).get(login, login)
+    
     @default
     @debug
     def authenticate(self, id=None, pw=None):
-        id = self.context._seckeys.get(
-            self.principal_attrmap.get('login'), {}).get(id, id)
+        # XXX: rename 'id' kw arg to 'login'
+        id = decode_utf8(id)
+        id = self.id_for_login(id)
         try:
-            userdn = self.context.child_dn(id)
+            if self.expiresAttr:
+                user = self.context[id]
+                expires = user.attrs.get(self.expiresAttr)
+                if expires and expires not in ['99999' '-1']:
+                    # check expiration timestamp
+                    try:
+                        expires = int(expires)
+                    except ValueError:
+                        # unknown expires field data
+                        msg= u"Accound expiration flag for user '%s' " +\
+                             u"contains unknown data"
+                        msg = msg % id
+                        logger.error(msg)
+                        return False
+                    
+                    # XXX: maybe configurable?
+                    # shadow account specific
+                    #if self.expiresAttr == 'shadowExpire':
+                    #    expires += int(user.attrs.get('shadowInactive', '0'))
+                    # /XXX
+                    
+                    days = time.time()
+                    if self.expiresUnit == EXPIRATION_DAYS:
+                        # numer of days since epoch
+                        days /= 86400
+                    if days >= expires:
+                        return ACCOUNT_EXPIRED
+                userdn = user.DN
+            else:
+                userdn = self.context.child_dn(id)
         except KeyError:
             return False
         return self.context.ldap_session.authenticate(userdn, pw) \
@@ -549,6 +663,7 @@ class LDAPUsers(LDAPPrincipals, UgmUsers):
     @default
     @debug
     def passwd(self, id, oldpw, newpw):
+        id = decode_utf8(id)
         self.context.ldap_session.passwd(
             self.context.child_dn(id), oldpw, newpw)
         object_classes = self.context.child_defaults['objectClass']
@@ -579,6 +694,8 @@ def member_format(obj_cl):
         return FORMAT_DN
     if 'posixGroup' in obj_cl:
         return FORMAT_UID
+    if 'group' in obj_cl:
+        return FORMAT_DN
     raise Exception(u"Unknown format")
 
 
@@ -589,6 +706,8 @@ def member_attribute(obj_cl):
         return 'uniqueMember'
     if 'posixGroup' in obj_cl:
         return 'memberUid'
+    if 'group' in obj_cl:
+        return 'member' # XXX: check AD!
     raise Exception(u"Unknown member attribute")
 
 
@@ -630,6 +749,7 @@ class LDAPGroups(LDAPGroupsMapping):
     @extend
     @locktree
     def __delitem__(self, key):
+        key = decode_utf8(key)
         group = self[key]
         parent = self.parent
         if parent and parent.rcfg is not None:
@@ -710,6 +830,7 @@ class LDAPRole(LDAPGroupMapping, AliasedPrincipal):
     @extend
     @locktree
     def __getitem__(self, key):
+        key = decode_utf8(key)
         if key not in self:
             raise KeyError(key)
         principals = self.related_principals(key)
@@ -720,6 +841,7 @@ class LDAPRole(LDAPGroupMapping, AliasedPrincipal):
     @extend
     @locktree
     def __delitem__(self, key):
+        key = decode_utf8(key)
         if key not in self:
             raise KeyError(key)
         principals = self.related_principals(key)
@@ -789,7 +911,7 @@ class LDAPUgm(UgmBase):
             GroupsConfig
         
         rcfg
-            RolesConfig XXX: not yet
+            RolesConfig
         """
         self.__name__ = name
         self.__parent__ = parent
